;; Generated package description from fj.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "fj" "0.37.0.20260509.74153" "Client for Forgejo instances" '((emacs "29.1") (fedi "0.2") (tp "0.8") (transient "0.10.0") (magit "4.3.8")) :commit "4832dcfcd167d1975bb44a024881c75fef64182e" :authors '(("Marty Hiatt" . "martianh@disroot.org")) :maintainer '("Marty Hiatt" . "martianh@disroot.org") :keywords '("git" "convenience") :url "https://codeberg.org/martianh/fj.el")
